﻿using System;
using System.IO;
using System.Runtime.InteropServices;
using System.ComponentModel;
using System.Windows.Forms;
using AddinExpress.MSO;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Microsoft.Office365.SharePoint.CoreServices.Extensions;
using Microsoft.Office365.SharePoint.FileServices;
using File = System.IO.File;
using Outlook = Microsoft.Office.Interop.Outlook;

namespace OnedriveOutlook
{
    /// <summary>
    ///   Add-in Express Add-in Module
    /// </summary>
    [GuidAttribute("3A351ECE-F060-4724-B4BF-A46B6B8691CE"), ProgId("OnedriveOutlook.AddinModule")]
    public partial class AddinModule : AddinExpress.MSO.ADXAddinModule
    {
        public IPagedCollection<IItem> Files = null;
        public Stream FileStream = null;
        public string AttachmentName = string.Empty;

        public int MESSAGE_FILES = 0x0400 + 903;
        public int MESSAGE_FILEDOWNLOAD = 0x0400 + 904;
        public AddinModule()
        {
            Application.EnableVisualStyles();
            InitializeComponent();
            // Please add any initialization code to the AddinInitialize event handler
        }

        #region Add-in Express automatic code

        // Required by Add-in Express - do not modify
        // the methods within this region

        public override System.ComponentModel.IContainer GetContainer()
        {
            if (components == null)
                components = new System.ComponentModel.Container();
            return components;
        }

        [ComRegisterFunctionAttribute]
        public static void AddinRegister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXRegister(t);
        }

        [ComUnregisterFunctionAttribute]
        public static void AddinUnregister(Type t)
        {
            AddinExpress.MSO.ADXAddinModule.ADXUnregister(t);
        }

        public override void UninstallControls()
        {
            base.UninstallControls();
        }

        #endregion

        public static new AddinModule CurrentInstance
        {
            get
            {
                return AddinExpress.MSO.ADXAddinModule.CurrentInstance as AddinModule;
            }
        }

        public Outlook._Application OutlookApp
        {
            get
            {
                return (HostApplication as Outlook._Application);
            }
        }

        private void AddinModule_OnSendMessage(object sender, ADXSendMessageEventArgs e)
        {
            if (e.Message == MESSAGE_FILES)
                ShowFiles();

            if (e.Message == MESSAGE_FILEDOWNLOAD)
                SaveAndAttachFile();
        }

        private void SaveAndAttachFile()
        {
            Outlook.Inspector currInspector = null;
            Outlook.MailItem currMail = null;
            Outlook.Attachments attachments = null;

            try
            {
                var fullFilePath = Path.Combine(Environment.GetFolderPath(Environment.SpecialFolder.MyDocuments), AttachmentName);
                var fs = File.Create(fullFilePath, (int)FileStream.Length);
                byte[] bytesInStream = new byte[this.FileStream.Length];
                this.FileStream.Read(bytesInStream, 0, bytesInStream.Length);
                fs.Write(bytesInStream, 0, bytesInStream.Length);
                fs.Close();

                currInspector = OutlookApp.ActiveInspector();
                currMail = currInspector.CurrentItem as Outlook.MailItem;
                attachments = currMail.Attachments;
                attachments.Add(fullFilePath, Outlook.OlAttachmentType.olByValue);

                var form = adxOlFormsCollectionItem1.GetCurrentForm() as frmOneDrive;
                form.btnAddAttachment.Enabled = true;
                form.panelStatus.Visible = false;

            }
            finally
            {
                if (attachments != null) Marshal.ReleaseComObject(attachments);
                if (currMail != null) Marshal.ReleaseComObject(currMail);
                if (currInspector != null) Marshal.ReleaseComObject(currInspector);
            }
        }

        private void ShowFiles()
        {
            var form = adxOlFormsCollectionItem1.GetCurrentForm() as frmOneDrive;
            form.ListFiles(Files.CurrentPage);
            form.picLoader.Visible = false;
            form.btnListFiles.Enabled = true;
        }
    }
}

