using System;
using System.Collections.Generic;
using System.Drawing;
using System.Threading.Tasks;
using System.Windows.Forms;
using System.Runtime.InteropServices;
using AddinExpress.OL;
using Microsoft.IdentityModel.Clients.ActiveDirectory;
using Microsoft.Office365.SharePoint.CoreServices;
using Microsoft.Office365.SharePoint.CoreServices.Extensions;
using Microsoft.Office365.SharePoint.FileServices;


namespace OnedriveOutlook
{
    public partial class frmOneDrive : AddinExpress.OL.ADXOlForm
    {
        private string clientId = "67fba0d2-8391-4ac3-af49-9dea87f4b400";
        private string authority = "https://login.microsoftonline.com/common";
        private string redirectUri = "http://localhost/0d303d2e84650b3efabfe6c4bda36cf8";
        private string resourceId = "https://{yourtenantid}-my.sharepoint.com/";
        private string serviceRoot = "https://{yourtenantid}-my.sharepoint.com/_api/v1.0/me";
        private string accessToken = string.Empty;
        private SharePointClient client = null;

        public AddinModule CurrentInstance
        {
            get
            {
                return this.AddinModule as AddinModule;
            }
        }
        public frmOneDrive()
        {
            InitializeComponent();
        }

        public void Authenticate()
        {
            var authContext = new AuthenticationContext(authority);
            var authResult = authContext.AcquireToken(resourceId, clientId,
                new Uri(redirectUri), PromptBehavior.Auto);

            accessToken = authResult.AccessToken;
        }

        private async void btnListFiles_Click(object sender, EventArgs e)
        {
            picLoader.Visible = true;
            btnListFiles.Enabled = false;

            CurrentInstance.Files = await GetFiles();
            CurrentInstance.SendMessage(CurrentInstance.MESSAGE_FILES);
        }

        private async Task<IPagedCollection<IItem>> GetFiles()
        {
            if (string.IsNullOrEmpty(accessToken))
                Authenticate();

            if (!string.IsNullOrEmpty(accessToken))
            {
                client = new SharePointClient(new Uri(serviceRoot), async () => accessToken);
                var fileResult = await client.Files.ExecuteAsync();
                return fileResult;
            }
            return null;
        }

        public void ListFiles(IReadOnlyList<IItem> currentPage)
        {
            foreach (var item in currentPage)
            {
                var node = new TreeNode(item.Name);
                node.Tag = item.Id;
                node.ImageKey = item.Type;
                node.SelectedImageKey = item.Type;
                tvFiles.Nodes.Add(node);
            }
        }

        private void tvFiles_NodeMouseClick(object sender, TreeNodeMouseClickEventArgs e)
        {
            btnAddAttachment.Visible = e.Node.ImageKey == "File";
        }

        private async void btnAddAttachment_Click(object sender, EventArgs e)
        {
            panelStatus.Visible = true;
            btnAddAttachment.Enabled = false;
            var fileId = tvFiles.SelectedNode.Tag.ToString();
            var fileFetcher = client.Files.GetById(fileId);
            var file = fileFetcher.ToFile();
            CurrentInstance.AttachmentName = tvFiles.SelectedNode.Text;
            CurrentInstance.FileStream = await file.DownloadAsync();
            CurrentInstance.SendMessage(CurrentInstance.MESSAGE_FILEDOWNLOAD);
        }
    }
}
